"""
Test for the thread return command.
"""

import unittest2
import lldb
import lldbutil
from lldbtest import *

class MultilevelThreadReturnTestCase(TestBase):

    mydir = os.path.join("thread_return", "")

    def setUp(self):
        TestBase.setUp(self)
        #traceback.print_stack()
        self.line = line_number('multi_level_return.c', "// Break here")
        self.line2 = line_number('multi_level_return.c', "// return here")
        self.exe_name = self.testMethodName

    @unittest2.skipUnless(sys.platform.startswith("darwin"), "requires Darwin")
    def test_with_dsym_multi(self):
        """ """
        d = {'C_SOURCES': 'multi_level_return.c', 'EXE': self.exe_name}
        self.buildDsym(dictionary=d)
        #self.plain_return_from_current_frame()
        self.thread_return2_with_type_and_value('int', 42)

    def test_with_dwarf_multi(self):
        """ """
        d = {'C_SOURCES': 'multi_level_return.c', 'EXE': self.exe_name}
        self.buildDwarf(dictionary=d)
        #self.plain_return_from_current_frame()
        self.thread_return2_with_type_and_value('int', 42)

    def thread_return2_with_type_and_value(self, type, value):
        self.runCmd("target create %s" % self.exe_name, CURRENT_EXECUTABLE_SET)

        self.expect("breakpoint set -f multi_level_return.c -l %d" % self.line, BREAKPOINT_CREATED,
            startstr = "Breakpoint created: 1: file ='multi_level_return.c', line = %d" % self.line)

        self.runCmd("run", RUN_SUCCEEDED)

        self.expect("frame return -f 1 -- (%s)(%d)" % (type, value), substrs = ['Returning: (%s)  = %d' % (type, value)])

        # BUG:
        self.runCmd("thread step-over", check=False) # The first one won't do anything
        self.runCmd("thread step-over") # This will.

        self.expect("frame variable r", None,
                    startstr = '(int) r = %d' % value)


if __name__ == '__main__':
    import atexit
    lldb.SBDebugger.Initialize()
    atexit.register(lambda: lldb.SBDebugger.Terminate())
    unittest2.main()
