"""
Test for the thread return command.
"""

import unittest2
import lldb
import lldbutil
from lldbtest import *

class BasicThreadReturnTestCase(TestBase):

    mydir = os.path.join("thread_return", "")

    def setUp(self):
        TestBase.setUp(self)
        #traceback.print_stack()
        self.line = line_number('basic_return.c', "// Break here to return another value")
        self.line2 = line_number('basic_return.c', "// We should be here after returning.") # Function call
        self.exe_name = self.testMethodName

    @unittest2.skipUnless(sys.platform.startswith("darwin"), "requires Darwin")
    def test_with_dsym_int(self):
        """ """
        d = {'C_SOURCES': 'int.c', 'EXE': self.exe_name}
        self.buildDsym(dictionary=d)
        #self.plain_return_from_current_frame()
        self.thread_return_with_type_and_value('int', 42)

    def test_with_dwarf_int(self):
        """ """
        d = {'C_SOURCES': 'int.c', 'EXE': self.exe_name}
        self.buildDwarf(dictionary=d)
        #self.plain_return_from_current_frame()
        self.thread_return_with_type_and_value('int', 42)

    @unittest2.skipUnless(sys.platform.startswith("darwin"), "requires Darwin")
    def test_with_dsym_char(self):
        """ """
        d = {'C_SOURCES': 'char.c', 'EXE': self.exe_name}
        self.buildDsym(dictionary=d)
        #self.plain_return_from_current_frame()
        self.thread_return_with_type_and_value('char', 42)

    def test_with_dwarf_char(self):
        """ """
        d = {'C_SOURCES': 'char.c', 'EXE': self.exe_name}
        self.buildDwarf(dictionary=d)
        #self.plain_return_from_current_frame()
        self.thread_return_with_type_and_value('char', 42)

    @unittest2.skipUnless(sys.platform.startswith("darwin"), "requires Darwin")
    def test_with_dsym_short(self):
        """ """
        d = {'C_SOURCES': 'short.c', 'EXE': self.exe_name}
        self.buildDsym(dictionary=d)
        #self.plain_return_from_current_frame()
        self.thread_return_with_type_and_value('short', 42)

    def test_with_dwarf_short(self):
        """ """
        d = {'C_SOURCES': 'short.c', 'EXE': self.exe_name}
        self.buildDwarf(dictionary=d)
        #self.plain_return_from_current_frame()
        self.thread_return_with_type_and_value('short', 42)

    @unittest2.skipUnless(sys.platform.startswith("darwin"), "requires Darwin")
    def test_with_dsym_unsigned(self):
        """ """
        d = {'C_SOURCES': 'unsigned.c', 'EXE': self.exe_name}
        self.buildDsym(dictionary=d)
        #self.plain_return_from_current_frame()
        self.thread_return_with_type_and_value('unsigned int', 42)

    def test_with_dwarf_unsigned(self):
        """ """
        d = {'C_SOURCES': 'unsigned.c', 'EXE': self.exe_name}
        self.buildDwarf(dictionary=d)
        #self.plain_return_from_current_frame()
        self.thread_return_with_type_and_value('unsigned int', 42)

    @unittest2.expectedFailure
    @unittest2.skipUnless(sys.platform.startswith("darwin"), "requires Darwin")
    def test_with_dsym_correct_pc(self):
        """ """
        d = {'C_SOURCES': 'int.c', 'EXE': self.exe_name}
        self.buildDsym(dictionary=d)
        self.return_from_current_frame_correct_pc()

    @unittest2.expectedFailure
    def test_with_dwarf_correct_pc(self):
        """ """
        d = {'C_SOURCES': 'int.c', 'EXE': self.exe_name}
        self.buildDwarf(dictionary=d)
        self.return_from_current_frame_correct_pc()

    # @unittest2.skipUnless(sys.platform.startswith("darwin"), "requires Darwin")
    # def test_with_dsym_correct_return(self):
    #     """ """
    #     self.buildDsym()
    #     self.return_from_current_frame_correct_return()

    # def test_with_dwarf_correct_return(self):
    #     """ """
    #     self.buildDwarf()
    #     self.return_from_current_frame_correct_return()

    # def tearDown(self):
    #     #traceback.print_stack()
    #     pass

    def plain_return_from_current_frame(self):
        """ """
        self.runCmd("target create %s" % self.exe_name, CURRENT_EXECUTABLE_SET)
        self.runCmd("log enable lldb expr")

        self.expect("breakpoint set -n function", BREAKPOINT_CREATED,
            startstr = "Breakpoint created: 1: name = 'function', locations = 1")

        self.runCmd("run", RUN_SUCCEEDED)

        self.expect("frame return 42", substrs = ['Returning: (int)  = 42'])

    def return_from_current_frame_correct_pc(self):
        """ """
        self.plain_return_from_current_frame()

        self.expect("register read pc", patterns = ['^[er]ip = 0x'],
                    substrs = ['%s`main + ' % self.exe_name,
                               'at main.c:%d' % self.line2])

    # def return_from_current_frame_correct_return(self):
    #     """ """
    #     self.plain_return_from_current_frame()

    #     # BUG:
    #     self.runCmd("thread step-over", check=False) # The first one won't do anything
    #     self.runCmd("thread step-over") # This will.

    #     self.expect("frame variable r", None,
    #                 startstr = '(int) r = 42')

    def get_type_format_string(self, type):
        if type == 'char':
            return "'%c'"
        else:
            return '%d'

    def thread_return_with_type_and_value(self, type, value):
        self.runCmd("target create %s" % self.exe_name, CURRENT_EXECUTABLE_SET)

        self.expect("breakpoint set -f basic_return.c -l %d" % self.line, BREAKPOINT_CREATED,
            startstr = "Breakpoint created: 1: file ='basic_return.c', line = %d" % self.line)

        self.runCmd("run", RUN_SUCCEEDED)

        self.expect("frame return 42")#, substrs = ['Returning: (%s)  = %d' % (type, value)])

        # BUG:
        self.runCmd("thread step-over", check=False) # The first one won't do anything
        self.runCmd("thread step-over") # This will.

        value_str = self.get_type_format_string(type) % value
        self.expect("frame variable r", None,
                    startstr = '(%s) r = %s' % (type, value_str))


if __name__ == '__main__':
    import atexit
    lldb.SBDebugger.Initialize()
    atexit.register(lambda: lldb.SBDebugger.Terminate())
    unittest2.main()
